import java.util.*;
import java.awt.*;
import java.awt.Graphics;
import java.awt.geom.*;
import java.applet.*;


public class GameObject {

	// state constants
	public static final int OBJ_OFF = 0;
	public static final int OBJ_ON = 1;
	// inheriting classes can define more constants
	
	private int state;
	private int x;
	private int y;
	private int velx;
	private int vely;
	private int angle;
	protected AffineTransform transform;
	private Graphics2D g;
	private AnimatedLists currentSequence;
	private HashMap<Integer, AnimatedLists> sequences;
	private Image [] imageSet;
	private int numOfStates;
	protected GameClass applet;
	public Random random;
	private boolean wrap;
	private int turnCount;
	private int turnSpeed;
	
	
	
	// the constructor for the GameObject
	// add stuff later. :)
	public GameObject(int imageSetSize, int numOfStates, Image [] imageFiles, int x, int y, Applet applet)
	{
		setImageSet(imageSetSize, imageFiles);
		sequences = new HashMap<Integer, AnimatedLists>(numOfStates);
		setState(OBJ_OFF);
		setX(x);
		setY(y);
		this.numOfStates = numOfStates;
		this.applet = (GameClass)applet;
		setAngle(90);
		random = new Random();
		wrap = true;
		clearTurn();
		
		
	}
	
	public boolean doesWrap()
	{
		return this.wrap;
	}
	
	public void setWrap(boolean wrap)
	{
		this.wrap = wrap;
	}
	
	public boolean createAnimatedLists(int state, int [] sequence, int speed)
	{

		if (this.sequences.size() < numOfStates)
			this.sequences.put(state, new AnimatedLists(sequence.length, sequence, speed));
		else
			return false;

		return true;

	}

	public boolean setAnimationSequence(int state)
	{
		
		// check that the state has a mapping in the hashmap
		if (this.sequences.containsKey(state))		
		{
			this.currentSequence = this.sequences.get(state);
			return true;
		}
		
		return false;

	}
	
	
	public void setImageSet(int size, Image [] imageFiles)
	{
		// set the imageset to something...
		// should get an array of stuff and ... yeah.
		imageSet = new Image[size];
		for (int i = 0; i < size; i++)
		{
			imageSet[i] = imageFiles[i];
		}
		
	}
	
	public int setImage(int index)
	{
		// set the current image of the GameObject
		// index into the Images array
		return index;
	}
	
	public void setOn()
	{
		// just sets the GameObject to alive
	}
	
	public void setOff()
	{
		// sets the GameObject to dead
	}
	
	// sets the x coordinate of the GameObject
	public void setX(int x)
	{

		this.x = x;
	}
	
	public void setX(double x)
	{
		setX((int) x);
	}
	
	// sets the y coordinate of the GameObject
	public void setY(int y)
	{
		this.y = y;
	}
	
	public void setY(double y)
	{
		setY((int) y);
	}
	
	/*
	 * sets the direction of motion of this GameObject
	 */
	public void setAngle(int angle)
	{
		this.angle = angle;
	}

	public void setVelX(int velx)
	{
		this.velx = velx;
	}

	public void setVelY(int vely)
	{
		this.vely = vely;
	}


  /*
   * update the position of the object
   */ 
	public void updatePos()
	{
		setX(getX() + getVelX());
		setY(getY() + getVelY());	

		if (wrap)
		{
			if (getX() - getWidth() > applet.getScreenWidth() && getVelX() > 0)
				setX(0);
			else if (getX() + getWidth() < 0 && getVelX() < 0)
				setX(applet.getScreenWidth());
			
			if (getY() - getHeight() > applet.getScreenHeight() && getVelY() > 0)
				setY(0);
			else if (getY() + getHeight() < 0 && getVelY() < 0)
				setY(applet.getScreenHeight());
		}
	}

	public void updateImage()
	{

		sequences.get(getState()).getNext();

	}
	
	public void setTransform(AffineTransform transform)
	{
		// set the transformation of this GameObject
	}
	
//	public void draw(Graphics2D g)
//	{
//		// draws the GameObject to the given Graphics device
//		g.drawImage(/* get the image here */ imageSet[currentSequence.getImagePos()], x, y, g);
//	}

	public Image getImage()
	{
		return imageSet[sequences.get(this.state).getImagePos()];
	}

	public int getPos()
	{
		return sequences.get(this.state).getPos();
	}
	
	public AnimatedLists getSequence()
	{
		return sequences.get(this.state);
	}
	
	public static boolean checkCollision(GameObject one, GameObject two)
	{
		// checks whether two GameObjects have collided. 
		// if their bounding rectangles intersect
		return one.getRectangle().intersects(two.getRectangle());
		
	}
	
	public Rectangle getRectangle()
	{
		//return new Rectangle(getX() + 10, getY() + 10, 
		//		imageSet[sequences.get(this.state).getImagePos()].getWidth(applet) - 10, 
		//		imageSet[sequences.get(this.state).getImagePos()].getHeight(applet) - 10);
		return new Rectangle(getX(), getY(), 
				imageSet[sequences.get(this.state).getImagePos()].getWidth(applet), 
				imageSet[sequences.get(this.state).getImagePos()].getHeight(applet));
	}

	public Rectangle getRectangle(int w, int h)
	{
		return new Rectangle(getX() + w, getY() + h, 
				imageSet[sequences.get(this.state).getImagePos()].getWidth(applet) - w, 
				imageSet[sequences.get(this.state).getImagePos()].getHeight(applet) - h);
	}

	/*
	 * sets the state of the GameObject
	 */
	public void setState(int state)
	{
		this.state = state;
	}
	
	/*
	 * returns the current state of the GameObject
	 */
	public int getState()
	{
		return this.state;
	}
	
	/*
	 * returns the current transformation to be used for
	 * drawing this GameObject
	 */
	public AffineTransform getTransform()
	{
		return null;
	}
	
	/*
	 * gets the direction of motion of this GameObject
	 */
	public int getAngle()
	{
		return this.angle;
	}
	
	/*
	 * returns the current y coordinate of the GameObject
	 */
	public int getY()
	{
		return this.y;
	}
	
	/*
	 * returns the current x coordinate of the GameObject
	 */
	public int getX()
	{
		return this.x;
	}

	public int getVelX()
	{
		return this.velx;
	}

	public int getVelY()
	{
		return this.vely;
	}
	
	public int getHeight()
	{
		return imageSet[sequences.get(this.state).getImagePos()].getHeight(applet);
	}
	
	public int getWidth()
	{
	//	System.out.println("state: " + this.state);
	//	if (sequences == null)
	//		System.out.println("we do contain the key man :S");
	//	else
	//		System.out.println("we don't contain it it or something :O");
	//	if (sequences.containsKey(this.state))
	//		System.out.println("we contain the key");
	//	else
	//		System.out.println("we really don't have this key man");

	//	System.out.println(sequences.toString());
		return imageSet[sequences.get(this.state).getImagePos()].getWidth(applet);
	}
	
	public void incTurn()
	{
		this.turnCount++;
	}
	
	public void clearTurn()
	{
		this.turnCount = 0;
	}
	
	public int checkTime()
	{
		return this.turnCount;
	}
	
	public void setTurnSpeed(int speed)
	{
		this.turnSpeed = speed;
	}
	
	public int getTurnSpeed()
	{
		return this.turnSpeed;
	}
	
	public void setRandomValuesAndUpdateAll()
	{
		if (random.nextDouble() > 0.9)
		{
			setVelX((int)(random.nextDouble() * 10 * (random.nextDouble() > 0.5 ? -1 : 1)));
			setVelY((int)(random.nextDouble() * 10 * (random.nextDouble() > 0.5 ? -1 : 1)));
		}
		//System.out.println("velx: " + getVelX());
		//System.out.println("vely: " + getVelY());
		updatePos();
		updateImage();
	}
	
	/*
	 * returns the current image of the GameObject
	 */
//	public Image getImage()
//	{
//		return null;
//	}
	
}
