import java.awt.*;
import java.applet.*;
import java.util.*;



public class Network {

	private GameObject [] stations;
	private int numOfStations;
	private int numOfClusters;
	//private final String station_off_1 = "images/station_off_1.png";
	//private final String station_off_2 = "images/station_off_2.png";
	//private final String station_on = "images/station_on.png";
	//private Image station_off_1;
	//private Image station_off_2;
	//private Image station_on;
	private Image [] station_imgs;
	private Applet applet;
	private int [][] station_coords;
	private ArrayList<EdgePipe> spanningTree;
	private EdgePipe [] pipes;
	private StationNode [] stationNodes;
	private GameObject tail;	// first selected
	private GameObject head;	// second selected
	
	
	public Network(int numOfClusters, int numOfStations, Image station_off_1, Image station_off_2, Image station_on, Applet applet)
	{
		this.numOfClusters = numOfClusters;
		this.numOfStations = numOfStations;
		stations = new GameObject[numOfStations * numOfClusters];
		//this.station_off_1 = station_off_1;
		//this.station_off_2 = station_off_2;
		//this.station_on = station_on;
		station_imgs = new Image[3];
		station_imgs[0] = station_off_1;
		station_imgs[1] = station_off_2;
		station_imgs[2] = station_on;
		this.applet = applet;
		this.stationNodes = new StationNode[numOfStations * numOfClusters];
		this.pipes = new EdgePipe[((numOfStations * (numOfStations - 1))/2)*numOfClusters 
		                          + numOfClusters - 1];

		this.head = null;
		this.tail = null;

		// creating the network 
		createNetwork();
		
		// creating the GameObjects for the stations
		createGameObjects();
				
	}

	// do this tomorrow... maybe write a method to retrieve a GameObject node by
	// location or something :)
	public boolean checkIfTree()
	{
		for (EdgePipe p : spanningTree)
		{
			if (!(getEdgeByNodes(getNodeByLocation(p.nodeA),
								getNodeByLocation(p.nodeB)).getState()))
			{
				return false;
			}
		}
		
		if (spanningTree.size() == getNumOfSelectedEdges())
			return true;
		else
			return false;
	}
	
	public int getNumOfSelectedEdges()
	{
		int num = 0;
		for (EdgePipe p : pipes)
		{
			if (p.getState())
				num++;
		}
		return num;
	}
	
	public void createNetwork()
	{
		// first column is x
		// second column is y
		station_coords = new int[numOfStations * numOfClusters][2];
		
		int width =  applet.getWidth();
		int height = applet.getHeight() / numOfClusters;
		
		Random random = new Random();
		
		boolean unique;
		int x = -1, y = -1;
		
		for (int i = 0; i < numOfClusters; i++)
		{
			for (int j = 0; j < numOfStations; j++)
			{
		
				unique = false;			
				
				while (!unique)
				{
					x = (int) (random.nextFloat() * 10) * 70;
					y = (int) ((random.nextFloat() * 5)) * 70;
					System.out.println("y: " + y);
					System.out.println("i: " + i);
					y +=  (height * (i));
					System.out.println("uniqe is false");
					unique = isUnique(numOfStations * i + j - 1, x, y);
				}				
				
				station_coords[numOfStations * i + j][0] = x;
				station_coords[numOfStations * i + j][1] = y;
				
				stationNodes[numOfStations * i + j] = new StationNode(x, y);
				
				System.out.println("x: " + x + "\ty: " + y);
			}
		}
		
		createPipes();
		createSpanningTree();
			
	}
	
	
	// used to make sure that stations do not overlap in the network
	public boolean isUnique(int until, int x, int y)
	{
		for (int i = 0; i <= until; i++)
		{
			if ((station_coords[i][0] == x) && (station_coords[i][1] == y))
				return false;
		}
		return true;
	}
	
	
	public void createNetwork2()
	{
		// first column is x
		// second column is y
		station_coords = new int[numOfStations * numOfClusters][2];
		
		int width =  applet.getWidth();
		int height = applet.getHeight() / numOfClusters;
		
		Random random = new Random();
		
		for (int i = 0; i < numOfClusters; i++)
		{
			for (int j = 0; j < numOfStations; j++)
			{
				int x = (int) (random.nextFloat() * width);
				int y = (int) ((random.nextFloat() * height));
				System.out.println("y: " + y);
				System.out.println("i: " + i);
				y +=  (height * (i));
				station_coords[numOfStations * i + j][0] = x;
				station_coords[numOfStations * i + j][1] = y;
				
				stationNodes[numOfStations * i + j] = new StationNode(x, y);
				
				System.out.println("x: " + x + "\ty: " + y);
			}
		}
		
		createPipes();
		createSpanningTree();
			
	}
	
	public void createPipes()
	{
		int current = 0;
		
		for (int c = 0; c < numOfClusters; c++)
		{
			for (int i = numOfStations * c; i < numOfStations * (c + 1); i++)
			{
				for (int j = i+1; j < numOfStations * (c + 1); j++)
				{
					pipes[current] = new EdgePipe(stationNodes[i], stationNodes[j]);
					current++;
				}
			}
		}
		
		for (int i = 0; i < numOfClusters - 1; i++)
		{
			pipes[current] = new EdgePipe(stationNodes[numOfStations * (i+1) - 1], 
										stationNodes[numOfStations * (i+1)]);
			current++;
		}
		
		EdgePipe temp;
		
		// sort the pipes using Selection Sort
		for (int i = 0; i < pipes.length - 1; i++)
		{
			for (int j = i + 1; j < pipes.length; j++)
			{
				if (pipes[j].getEdgeLength() < pipes[i].getEdgeLength())
				{
					temp = pipes[j];
					pipes[j] = pipes[i];
					pipes[i] = temp;
				}
			}
		}
		
		
		for (EdgePipe e : pipes)
		{
			System.out.println("length: " + e.getEdgeLength());
		}
		
	}
	
	public void createSpanningTree()
	{
		// pipes
		
		spanningTree = new ArrayList<EdgePipe>();
		
		spanningTree.add(pipes[0]);
		pipes[0].nodeA.setState(true);
		pipes[0].nodeB.setState(true);
		
		//for (int c = 0; c < 5; c++)
		while (!spannedTree())
		{
					
			System.out.println("still creating the spanning tree");
			for (int i = 1; i < pipes.length; i++)
			{
				
				// add this edge to the ArrayList spanningTree if at least one of the nodes
				// is not yet in the tree
				//if (!pipes[i].nodeA.getState() || !pipes[i].nodeB.getState())
				if ((pipes[i].nodeA.getState() && !pipes[i].nodeB.getState())
						|| (!pipes[i].nodeA.getState() && pipes[i].nodeB.getState()))
				{
					System.out.println("the fish are JUMPING and the cotton is high");
					spanningTree.add(pipes[i]);
					pipes[i].nodeA.setState(true);
					pipes[i].nodeB.setState(true);
					i = 0;
				}
			}
		}
			
		System.out.println("There are this many edges in the tree" + spanningTree.size());
		
	}
	
	public boolean spannedTree()
	{
		for (StationNode station : stationNodes)
		{
			if (!station.getState())
				return false;
		}
		
		return true;
		
	}
	
	
	
	public void createGameObjects()
	{
		for (int i = 0; i < numOfStations * numOfClusters; i++)
		{
			
			// imageSetSize, numOfStates, Image [] imagefiles, int x, int y, Applet applet
			// the states are : red, green
			stations[i] = new GameObject(3, 3, station_imgs, station_coords[i][0], station_coords[i][1], this.applet);
			stations[i].createAnimatedLists(GameObject.OBJ_ON, new int [] {0, 1}, 500);
			stations[i].createAnimatedLists(GameObject.OBJ_OFF, new int [] {2}, 500);
			stations[i].createAnimatedLists(555, new int[] {2, 2, 2, 1, 1, 1}, 500);
			
		}
	}
	
	public GameObject [] getStations()
	{
		return stations;
	}
	
	public void updateImages()
	{
		for (GameObject g : stations)
		{
			g.updateImage();
		}
	}
	
	public void createNewestNetwork(int numOfClusters, int numOfStations)
	{
		if ((this.numOfClusters != numOfClusters) || 
				(this.numOfStations != numOfStations))
		{
			this.numOfClusters = numOfClusters;
			this.numOfStations = numOfStations;	                          
		}
	}


	public void checkCollisions(GameObject one)
	{
		
		StationNode a;
		StationNode b;

		for (GameObject g : getStations())
		{
			//if (GameObject.checkCollision(one, g))
			if (one.getRectangle().contains((double)g.getX(), (double)g.getY(), (double)g.getWidth(), (double)g.getHeight()))
			{
				System.out.println("intersection\n");
				//g.setState(GameObject.OBJ_OFF);
				if ((tail == null))
				{
					tail = g;
					//if (g.getState() == GameObject.OBJ_ON)
					g.setState(555);
				}
 				else if (tail == g)
 				{
 					if (getStationNodeByLocation(g).selected() == 0)
 						g.setState(GameObject.OBJ_ON);
 					else
 						g.setState(GameObject.OBJ_OFF);
 					tail = null;
 				}
 				else if (getEdgeByNodes(tail, g) == null)
 				{
 					if (getStationNodeByLocation(tail).selected() == 0)
 						tail.setState(GameObject.OBJ_ON);
 					else
 						tail.setState(GameObject.OBJ_OFF);
 						
 					tail = g;
 					tail.setState(555);
 				}
 				else
 				{
					head = g;
										
					a = getStationNodeByLocation(tail);
					b = getStationNodeByLocation(head);
				
					if (getEdgeByNodes(tail, head).getState())
					{
						// we unselect this edge
						a.decSelect();
						b.decSelect();
						if (a.selected() == 0)
							tail.setState(GameObject.OBJ_ON);
						else
							tail.setState(GameObject.OBJ_OFF);
						if (b.selected() == 0)
							head.setState(GameObject.OBJ_ON);
						else
							head.setState(GameObject.OBJ_OFF);
						
						getEdgeByNodes(tail, head).setState(false);
						System.out.println("state: " + getEdgeByNodes(tail, head).getState());
					}
					else
					{
						// we select this edge
						a.incSelect();
						b.incSelect();
						tail.setState(GameObject.OBJ_OFF);
						head.setState(GameObject.OBJ_OFF);
						getEdgeByNodes(tail, head).setState(true);
					}
					
					head = null;
					tail = null;
				}

			}
		}


	}

	public EdgePipe getEdgeByNodes(GameObject one, GameObject two)
	{
		for (EdgePipe p : pipes)
		{
			if (((one.getX() == p.nodeA.x && one.getY() == p.nodeA.y)
					&& (two.getX() == p.nodeB.x && two.getY() == p.nodeB.y))
					|| ((one.getX() == p.nodeB.x && one.getY() == p.nodeB.y)
					&& (two.getX() == p.nodeA.x && two.getY() == p.nodeA.y)))
			{
				return p;
			}
		}
		
		return null;
	}
	
	public StationNode getStationNodeByLocation(GameObject one)
	{
		for (StationNode s : stationNodes)
		{
			if (s.x == one.getX() && s.y == one.getY())
				return s;
		}
		
		return null;
	}
	
	public GameObject getNodeByLocation(StationNode node)
	{
		for (GameObject s : stations)
		{
			if (s.getX() == node.x && s.getY() == node.y)
				return s;
		}
		
		return null;
	}
	


	
	public void draw()
	{
	
		Graphics2D g2d = ((GameClass)applet).g2d; //(Graphics2D)((Foo)applet).getGraphics();
		//System.out.println("hehllo'");
		//if (applet == null)
		//	System.out.println("applet is null :(");
		
		//if (g2d == null)
		//{
		//	System.out.println("applet is null :(");
		//	return;
		//}
		
		g2d.setStroke(new BasicStroke(5.0F));
		g2d.setColor(Color.BLUE);
		
		
//		for (int c = 0; c < numOfClusters; c++)
//		{
//			for (int i = 0 + c*numOfStations; i < (c+1)*numOfStations; i++)
//			{
//				for (int j = i+1; j < (c+1)*numOfStations; j++)
//				{
//					g2d.drawLine(stations[i].getX() + stations[i].getWidth()/2, stations[i].getY() + stations[i].getHeight()/2, stations[j].getX() + stations[j].getWidth()/2, stations[j].getY() + stations[j].getHeight()/2);
//				}
//			}
//		}
		
		// draw the unselected edges first and then overlap the red (selected ones)
		// for the cases with 3+ collinear nodes
		
		g2d.setColor(Color.BLUE);
		for (EdgePipe p : pipes)
		{
			g2d.drawLine(p.nodeA.x + stations[0].getWidth()/2, p.nodeA.y + stations[0].getHeight()/2, p.nodeB.x + stations[0].getWidth()/2, p.nodeB.y + stations[0].getHeight()/2);
		}
		
		g2d.setColor(Color.GREEN);
		for (EdgePipe p : pipes)
		{
			if (p.getState())
				g2d.drawLine(p.nodeA.x + stations[0].getWidth()/2, p.nodeA.y + stations[0].getHeight()/2, p.nodeB.x + stations[0].getWidth()/2, p.nodeB.y + stations[0].getHeight()/2);
		}
//		for (EdgePipe p : pipes)
//		{
//			if (p.getState())		
//				g2d.setColor(Color.RED);
//			else
//				g2d.setColor(Color.BLUE);
//			g2d.drawLine(p.nodeA.x + stations[0].getWidth()/2, p.nodeA.y + stations[0].getHeight()/2, p.nodeB.x + stations[0].getWidth()/2, p.nodeB.y + stations[0].getHeight()/2);
//			
//			//System.out.println("state: " + p.getState());
//		}
		
		//for (int c = 0; c < numOfClusters - 1; c++)
		//{
		//	for (int i = c*numOfStations; i < (c+1)*numOfStations; i++)
		//		g2d.drawLine(stations)
		//}
		
		for (int i = 0; i < numOfClusters * numOfStations; i++)
		{
			g2d.drawImage(getStations()[i].getImage(), getStations()[i].getX(), getStations()[i].getY(), applet);
		}
		
//		g2d.setColor(Color.RED);
//		
//		for (EdgePipe e : spanningTree)
//		{
//			g2d.drawLine(e.nodeA.x, e.nodeA.y, e.nodeB.x, e.nodeB.y);
//		}
		
		
		
		
		
	}
	
	
	
}


class StationNode
{
	
	public int x;
	public int y;
	private boolean state;
	private int playerSelect;
	
	public StationNode(int x, int y)
	{
		this.x = x;
		this.y = y;
		this.state = false;
		this.playerSelect = 0;
	}
	
	public void setState(boolean state)
	{
		this.state = state;
	}
	
	public boolean getState()
	{
		return this.state;
	}
	
	public int selected()
	{
		return this.playerSelect;
	}
	
	public void incSelect()
	{
		this.playerSelect++;
	}
	
	public void decSelect()
	{
		this.playerSelect--;
	}
	
	
}

class EdgePipe
{
	
	public StationNode nodeA;
	public StationNode nodeB;
	private double edgeLength;
	private boolean state;
	
	public EdgePipe(StationNode a, StationNode b)
	{
		this.nodeA = a;
		this.nodeB = b;
		this.edgeLength = Math.sqrt((b.x - a.x) * (b.x - a.x) + (b.y - a.y) * (b.y - a.y));
		this.state = false;
	}
	
	public double getEdgeLength()
	{
		return this.edgeLength;
	}
	
	public boolean getState()
	{
		return this.state;
	}
	
	public void setState(boolean state)
	{
		this.state = state;
	}
}

